#!/bin/bash
#
# 08/10/2013 Updated to add detection for tomcat verstion (RA)
# 14/02/2014 Updated to copy cache.xml from old location to new config dir (RA)

source /backups/patch/settings.sh

cleanup() {
 cd $PATCHBASE
 rm -rf *
 rm -rf /tmp/proxy
 rm -rf /tmp/userportal
 rm -rf /tmp/pinsafe
 echo; echo "Clean up done..."
}

initialise() {
 if test -f /etc/rc.d/init.d/tomcat5; then
  TOMCAT=tomcat5; fi
 if test -f /etc/rc.d/init.d/tomcat; then
  TOMCAT=tomcat; fi
 if test ! $TOMCAT; then
  echo "Unable to detect the version of tomcat installed."
  echo "Exiting..."
  exit; fi
}

appliance_type() {
 PINSAFE_TYPE=`grep PINSAFE_TYPE $PINSAFE_CONF|cut -d\= -f2`

 if test ! $PINSAFE_TYPE; then
  stnd_err "Appliance type is not correctly configured."; fi
 if test "$PINSAFE_TYPE" = "Single"; then
  PINSAFE_TYPE="Single "; fi
}

check_drbd() {
 DRBD_TYPE=`grep DRBD $PINSAFE_CONF|cut -d\= -f2`
 
 if test "$DRBD_TYPE" = "yes"; then
   SWIVEL=$SWIVEL_DRBD
   USER_PORTAL=$USER_PORTAL_DRBD
   USER_PORTAL_OLD=$USER_PORTAL_DRBD_OLD
 fi
}

heading() {
 clear
 echo "${inv} Swivel Maintenance (c) $COPYRIGHT_YEAR${nor}                                             ${inv}$PINSAFE_TYPE${nor}"
 echo
 echo $1
 echo
}

execute_if_false() {
 echo
 echo -n "${2:-Are you sure}? (Yes/No): "
 read yn
 check_if_false "$1" "$yn"
}

execute_if_true() {
 echo
 echo -n "${2:-Are you sure}? (Yes/No): "
 read yn
 check_if_true "$1" "$yn"
}

check_if_false() {
 if test ! $2; then
  $1
  else
 if test ! $2 = "Yes" && test ! $2 = "yes" && test ! $2 = "y" && test ! $2 = "Y" ; then
  $1; fi
  fi
}

check_if_true() {
 if test ! $2; then
  $1
  else
 if test $2 = "Yes" || test $2 = "yes" || test $2 = "y" || test $2 = "Y" ; then
  $1; fi
  fi
}

check_running() {
 if test "`ps -eaf|grep $2|egrep -v "UID|grep|mysqld_safe|alert.d" 2>/dev/null`"; then
  if test ! -f $LOCK/$1; then
   touch $LOCK/$1; fi
 else
  if test -f $LOCK/$1; then
   rm -f $LOCK/$1; fi
 fi

 if test -f $LOCK/$1; then
  value="Running"
 else
  value="Stopped"
 fi
}

show_dots() {
 i=0
 while test $i -lt $1; do
  echo -n "."
  sleep 1 
  ((i = i + 1))
 done
}

service_sands_tomcat() {
 nohup service $TOMCAT $1 > /dev/null 2>&1 &
 echo -n "Tomcat: $1"
 show_dots 4
 echo
}

service_sands() {
 service $1 $2
}

warning() {
 heading "Upgrading Swivel Applications"
 echo "Continuing will apply patch ${inv}Swivel Combined Apps $PATCH_VER${nor}."
 echo "The Swivel solution cannot be used whilst upgrading."
 echo
 execute_if_false exit
}

appliance_version() {
 APPLIANCE_VERS=`grep ^APPLIANCE_VERS= $PINSAFE_CONF|cut -d\= -f2`

 if test $APPLIANCE_VERS && test $APPLIANCE_REQUIRED; then
  if ! version_ge $APPLIANCE_VERS $APPLIANCE_REQUIRED; then
   echo "The appliance requires patching before Swivel can be updated"
   echo; read -p "Press Return to exit" yn
   exit; fi
 else
  echo "Unable to determine appliance version"
  echo; read -p "Press Return to exit" yn
  exit
 fi
}

version_ge() {
  # return 0 iff version1 is greater or equal to version2
  MAJ1=$(echo $1 | cut -d. -f1);
  MIN1=$(echo $1 | cut -d. -f2);
  MIC1=$(echo $1 | cut -d. -f3);

  MAJ2=$(echo $2 | cut -d. -f1);
  MIN2=$(echo $2 | cut -d. -f2);
  MIC2=$(echo $2 | cut -d. -f3);
  if [ $MAJ1 -gt $MAJ2 ]; then return 0; fi
  if [ $MAJ1 -lt $MAJ2 ]; then return 1; fi

  if [ $MIN1 -gt $MIN2 ]; then return 0; fi
  if [ $MIN1 -lt $MIN2 ]; then return 1; fi

  if [ $MIC1 -gt $MIC2 ]; then return 0; fi
  if [ $MIC1 -lt $MIC2 ]; then return 1; fi
  if [ $MIC1 -eq $MIC2 ]; then return 0; fi
  return 1;
}

update_values() {
 sed -i "s/PINSAFE_PATCH_VER=/PINSAFE_PATCH_VER=$PATCH_VER/g" $PINSAFE_CONF 
}

update_firewall() {
 if test ! "`grep $SWIVEL_PORT $IPTABLES`"; then
  echo "Updating Appliance firewall to enable Swivel synchronisation"
  sed -i '/--dport 10000/a-A RH-Firewall-1-INPUT -m state --state NEW -m tcp -p tcp --dport 61616 -j ACCEPT' $IPTABLES
 # filesize=`cat $IPTABLES|wc -l`
#  ((linestoextract = $filesize -2))

#  head -$linestoextract $IPTABLES>/tmp/iptables.$$
# echo "-A RH-Firewall-1-INPUT -m state --state NEW -m tcp -p tcp --dport $SWIVEL_PORT -j ACCEPT">>/tmp/iptables.$$
# tail -2 $IPTABLES>>/tmp/iptables.$$
# cat /tmp/iptables.$$ > $IPTABLES
# rm /tmp/iptables.$$
  service iptables restart
 fi
}

configure_image_url() {
	echo
	echo "Please enter the server details for displaying TURing images in the User Portal:"
	echo
	echo "Use https (Y/N)?"
	read yn
	if test ! $yn = "Yes" && test ! $yn = "yes" && test ! $yn = "y" && test ! $yn = "Y" ; then
		swivel_https='false'
	else
		swivel_https='true'
	fi
	echo
	echo "Please enter the server name or IP address:"
	read swivel_server
	echo
	echo "Please enter the server port (e.g. 8080 or 8443):"
	read swivel_port
	echo
	echo "Please enter the server context (e.g. pinsafe or proxy):"
	read swivel_context
	
	# OK, we've got all the values, let's insert them into the properties file.
	# We're assuming the original properties file is passed as the function input
	# and we will echo the modified properties file.
	
	sed -e "s/imagessl=.*/imagessl=${swivel_https}/" -e "s/imageserver=.*/imageserver=${swivel_server}/" -e "s/imagecontext=.*/imagecontext=${swivel_context}/" -e"s/imageport=.*/imageport=${swivel_port}/" $1 > $2
}

archive_and_remove() {
 if test -z $1; then
   echo "No application specified"; return; fi
 
 echo; echo "Creating backup of $1 to /tmp"
 if test -d $TOMCAT_PATH/webapps2/$1; then
  cp -rp $TOMCAT_PATH/webapps2/$1 /tmp; 
 echo; echo "Removing old application, please wait..."
 rm -f $TOMCAT_PATH/webapps2/$1.war
 sleep 10
 if test -d $TOMCAT_PATH/webapps2/$1; then
   sleep 10
   if test -d $TOMCAT_PATH/webapps2/$1; then
    rm -rf $TOMCAT_PATH/webapps2/$1;
	fi 
 fi
  
 fi
}

check_remove_app() {
	if test -d $TOMCAT_PATH/webapps2/$1; then
	  execute_if_true "archive_and_remove $1" "Remove $1 application"
	fi
}

remove_old_apps() {
	check_remove_app 'resetpin'
	check_remove_app 'changepin'
}

backup_old_versions() {
 check_running $TOMCAT tomcat
 if test $value = "Running"; then
  echo "Stopping Tomcat and Swivel service"
  service_sands $TOMCAT stop
 fi
 
 echo "Creating new Swivel configuration dir."
 mkdir -p $SWIVEL
 chown -R swivel:swivel $SWIVEL
 rm -f /opt/.swivel
 ln -s /home/swivel/.swivel /opt/.swivel
 sed -i '/SWIVEL_HOME/cexport SWIVEL_HOME=/home/swivel/.swivel' /etc/profile.d/swivel.sh
 export SWIVEL_HOME=/home/swivel/.swivel

 echo "Creating backups of original configurations"
 if test -d $TOMCAT_PATH/webapps/pinsafe/WEB-INF/conf; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/conf $SWIVEL; fi
 if test -d $TOMCAT_PATH/webapps/pinsafe/WEB-INF/data; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/data $SWIVEL; fi
 if test -d $TOMCAT_PATH/webapps/pinsafe/WEB-INF/db; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/db $SWIVEL; fi
 if test -d $TOMCAT_PATH/webapps/pinsafe/WEB-INF/logs; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/logs $SWIVEL; fi
 if test -d $TOMCAT_PATH/webapps/pinsafe/WEB-INF/active-mq; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/active-mq $SWIVEL; fi
 if test -d $TOMCAT_PATH/webapps/pinsafe/WEB-INF/reports; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/reports $SWIVEL; fi
 if test -f $TOMCAT_PATH/webapps/pinsafe/WEB-INF/classes/cache.xml; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe/WEB-INF/classes/cache.xml $SWIVEL/conf; fi

 echo "Creating backup of original Swivel core to /tmp"
 if test -d $TOMCAT_PATH/webapps/pinsafe; then
  cp -rp $TOMCAT_PATH/webapps/pinsafe /tmp; fi
 
 echo "Creating backup of original Swivel User Portal to /tmp"
 if test -d $TOMCAT_PATH/webapps2/userportal; then
  cp -rp $TOMCAT_PATH/webapps2/userportal /tmp; fi

  echo "Creating backup of original Swivel proxy to /tmp"
 if test -d $TOMCAT_PATH/webapps2/proxy; then
  cp -rp $TOMCAT_PATH/webapps2/proxy /tmp; fi

if test $NEW_DIR = 1 && test -d $USER_PORTAL_OLD; then
 echo "Creating new User Portal configuration dir."
 mkdir -p $USER_PORTAL
 chown -R swivel:swivel $USER_PORTAL

 for file in $USER_PORTAL_OLD/*; do
  mv $file $USER_PORTAL	
 done
fi

 echo "Starting Tomcat and Swivel service"
 service_sands_tomcat start
}

check_old_versions_removed() {
 RETVAL=0
 if test -d $TOMCAT_PATH/webapps/pinsafe; then
  RETVAL=1
  return
 fi
 if test -d $TOMCAT_PATH/webapps2/userportal; then
  RETVAL=1
  return
 fi
 if test -d $TOMCAT_PATH/webapps2/proxy; then
  RETVAL=1
 fi
}

force_remove_old_versions() {
 if test -d $TOMCAT_PATH/webapps/pinsafe; then
  rm -rf $TOMCAT_PATH/webapps/pinsafe
 fi
 if test -d $TOMCAT_PATH/webapps2/userportal; then
   rm -rf $TOMCAT_PATH/webapps2/userportal
 fi
 if test -d $TOMCAT_PATH/webapps2/proxy; then
   rm -rf $TOMCAT_PATH/webapps2/proxy
 fi
}

remove_old_versions() {
 echo; echo "Removing old Swivel Core instance, please wait..."
 rm -f $TOMCAT_PATH/webapps/pinsafe.war
 sleep 5

 echo; echo "Removing old user portal instance, please wait..."
 rm -f $TOMCAT_PATH/webapps2/userportal.war
 sleep 5

 echo; echo "Removing old proxy instance, please wait..."
 rm -f $TOMCAT_PATH/webapps2/proxy.war
 sleep 5

 check_old_versions_removed
 if test $RETVAL = 1; then
  sleep 10
  check_old_versions_removed
  if test $RETVAL = 1; then
   sleep 10
   force_remove_old_versions
  fi
 fi  
}

check_new_versions_installed() {
 RETVAL=0
 if test ! -d $TOMCAT_PATH/webapps/pinsafe; then
  RETVAL=1
  return
 fi
 if test ! -d $TOMCAT_PATH/webapps2/userportal; then
  RETVAL=1
  return
 fi
 if test ! -d $TOMCAT_PATH/webapps2/proxy; then
  RETVAL=1
 fi
}

install_new_versions() {

 chown swivel:swivel $PATCHBASE/pinsafe.war $PATCHBASE/userportal.war $PATCHBASE/proxy.war

 echo; echo "Deploying new Swivel core software"
 cp -p $PATCHBASE/pinsafe.war $TOMCAT_PATH/webapps 

 echo; echo "Deploying new Swivel User Portal software"
 cp -p $PATCHBASE/userportal.war $TOMCAT_PATH/webapps2 

 echo; echo "Deploying new Swivel Proxy software"
 cp -p $PATCHBASE/proxy.war $TOMCAT_PATH/webapps2 

 idx=0
 RETVAL=1
 while test $idx -lt 12 && test $RETVAL = 1; do
   sleep 5
   check_new_versions_installed
   idx=$(($idx+1))
 done
 if test $RETVAL = 1; then
   # Give up!
   echo "Some applications failed to deploy"
   echo "Restarting tomcat"
   service_sands_tomcat restart
 else
   echo "All applications installed"
 fi
}

appliance_type
heading "Upgrading Swivel Applications"
initialise
appliance_version
check_drbd
warning
NEW_DIR=0
if test ! -d $USER_PORTAL; then
	NEW_DIR=1
fi
SETPROPS=1
if test -f $USER_PORTAL/settings.properties; then
	SETPROPS=0
else
  if test -f $TOMCAT_PATH/webapps2/userportal/WEB-INF/classes/settings.properties; then
    if test ! -d $USER_PORTAL; then
      mkdir $USER_PORTAL
    fi
    cp $TOMCAT_PATH/webapps2/userportal/WEB-INF/classes/settings.properties $USER_PORTAL/.
	SETPROPS=0
  fi
fi
backup_old_versions
remove_old_versions
install_new_versions
if test -f $TOMCAT_PATH/webapps2/userportal/WEB-INF/classes/settings.properties && test $SETPROPS = 1; then
  echo "Preparing to configure portal..."
  service_sands_tomcat stop
  if test ! -d $USER_PORTAL; then
    mkdir $USER_PORTAL
  fi
  configure_image_url $TOMCAT_PATH/webapps2/userportal/WEB-INF/classes/settings.properties $USER_PORTAL/settings.properties
  service_sands_tomcat start 
fi
if test -d $USER_PORTAL_OLD; then
 rm -rf $USER_PORTAL_OLD
fi
remove_old_apps
update_values
update_firewall
cleanup
echo; read -p "Press Return to continue" yn
